package info.lotosummary.software;
import java.io.*;

public class Queue {
    static final char[] BLANK = {' '};
    static final char[] NOT_APPLICABLE = {'n', '/', 'a'};
    static final int EMPTY_ERROR = 2;
    static final int FULL_HOUSE_ERROR = 3;
    static final int QUEUE_ARRAY_SIZE = 4;
    static final int QUEUE_ERROR_COUNT = 5;
    static final int QUEUE_GROWN = 0;
    static final int QUEUE_SHRUNK = 1;

    String[] queue;
    int n, queue_begin, queue_end;

    private int next(int index) {
        n = (index + 1) % QUEUE_ARRAY_SIZE;
        return n;
    }

    public boolean empty() {
        boolean b = false;

        if(queue_begin == queue_end) {
            b = true;
        }

        return b;
    }

    public int display() {
        int item;
        int size = queue_size();

        item = queue_begin;
        for(int i = 0; i < size; ++i) {
            System.out.println(queue[item]);
            item = next(item);
        }

        return 0;
    }

    public int pop_front() {
        if( empty() ) {
            return EMPTY_ERROR;
        }

        queue[queue_begin] = new String(NOT_APPLICABLE);
        queue_begin = next(queue_begin);
        return QUEUE_SHRUNK;
    }

    public int produce_ring_buffer(int size) {
        clear();

        for(int i = 1; i <= size; ++i) {
            push_back(BLANK);
        }

        return queue_size();
    }

    public int push_back(char[] p) {
        if( next(queue_end) == queue_begin ) {
            return FULL_HOUSE_ERROR;
        }

        queue[queue_end] = new String(p);
        queue_end = next(queue_end);
        return QUEUE_GROWN;
    }

    public int push_back(String s) {
        if( next(queue_end) == queue_begin ) {
            return FULL_HOUSE_ERROR;
        }

        queue[queue_end] = new String(s);
        queue_end = next(queue_end);
        return QUEUE_GROWN;
    }

    public int queue_size() {
        int size;

        if(queue_begin < queue_end) {
            size = queue_end - queue_begin;
        } else if(queue_begin == queue_end) {
            size = 0;
        } else {
            size = queue_end + QUEUE_ARRAY_SIZE - queue_begin;
        }

        return size;
    }

    public Queue() {
        queue = new String[QUEUE_ARRAY_SIZE];
    }

    public String chain() {
        int item;
        int size = queue_size();
        StringBuffer os = new StringBuffer();

        item = queue_begin;
        for(int i = 0; i < size; ++i) {
            os.append(queue[item]);
            item = next(item);
        }

        String s = os.toString();
        return s;
    }

    public String chain(String[] separatorList) {
        int item;
        int separatorListLength = separatorList.length;
        int size = queue_size();
        StringBuffer os = new StringBuffer();

        item = queue_begin;
        if(separatorListLength >= size) {
            for(int i = 0; i < size; ++i) {
                os.append(queue[item]);
                os.append(separatorList[i]);
                item = next(item);
            }

        } else {
            int k;
            for(k = 0; k < separatorListLength; ++k) {
                os.append(queue[item]);
                os.append(separatorList[k]);
                item = next(item);
            }

            for( ; k < size; ++k) {
                os.append(queue[item]);
                item = next(item);
            }

        }

        String s = os.toString();
        return s;
    }

    public String front() {
        return queue[queue_begin];
    }

    public void clear() {
        int k;
        int repetition = queue_size();
        for(k = 0; k < repetition; k++) {
            pop_front();
        }

    }

}
